package de.unisaar.jfap.ex5.task1;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Set;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Provide method to find marker in data stream.
 *
 * @author Jörg Steffen, DFKI
 */
public final class MarkerFinder {

  private static final Logger logger = LoggerFactory.getLogger(MarkerFinder.class);

  private MarkerFinder() {
    // private constructor to enforce noninstantiability
  }

  /**
   * Read string from given path and find start-of-packet marker. Marker is defined by a sequence of
   * four different characters.
   *
   * @param path the file to read the data stream from
   * @return the index position in the data stream after the first marker or {@code -1} if marker is
   *     not found
   * @throws IOException if reading file fails
   */
  public static int find(Path path) throws IOException {
    String input = Files.readString(path);
    Queue<Character> queue = new LinkedList<>();
    char[] chars = input.toCharArray();
    Set<Character> window = new HashSet<>();
    int markerSize = 4;
    // add first 3 characters to queue
    for (int i = 0; i < markerSize - 1; i++) {
      queue.add(chars[i]);
    }
    for (int i = markerSize - 1; i < chars.length; i++) {
      // add next character to queue
      queue.add(chars[i]);
      // check for duplicates
      window.clear();
      window.addAll(queue);
      if (window.size() == markerSize) {
        // all values are unique now
        logger.info("first marker after character {}", i + 1);
        return i + 1;
      }
      // remove head of queue
      queue.poll();
    }
    logger.info("no marker found");
    return -1;
  }
}
