package de.unisaar.jfap.ex5.task2;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Provide method to simulate Bingo game.
 *
 * @author Jörg Steffen, DFKI
 */
public final class Bingo {

  private static final Logger logger = LoggerFactory.getLogger(Bingo.class);

  private Bingo() {
    // private constructor to enforce noninstantiability
  }

  private static List<Board> initBoards(List<String> lines) {
    List<Board> boards = new ArrayList<>();
    for (int i = 2; i < lines.size(); i = i + 6) {
      List<String> boardLines = lines.subList(i, i + 5);
      Board board = new Board(boardLines);
      boards.add(board);
    }
    return boards;
  }

  /**
   * Simulate Bingo game using the input read from the given file.
   *
   * @param path the file to read the input from
   * @return the score of the winning board or {@code 0} if no board wins
   * @throws IOException if reading input fails
   */
  public static int play(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);
    List<Board> boards = initBoards(lines);
    for (Board oneBoard : boards) {
      System.out.println(oneBoard);
    }

    // get numbers
    List<Integer> numbers =
        Arrays.stream(lines.get(0).split(",")).map(s -> Integer.parseInt(s)).toList();

    // process numbers
    for (int oneNum : numbers) {
      for (Board oneBoard : boards) {
        oneBoard.checkNumber(oneNum);
        if (oneBoard.hasWon()) {
          int finalScore = oneBoard.getUnmarkedNumbersSum() + oneNum;
          logger.info(String.format("winning board with last number %d:%n%s", oneNum, oneBoard));
          logger.info("final score {}", finalScore);
          return finalScore;
        }
      }
    }

    // no winner
    logger.info("no board winning after all numbers have been drawn");
    return 0;
  }
}
