package de.unisaar.jfap.ex5.task2;

import java.util.List;

/**
 * Board for playing Bingo with numbers and marks.
 *
 * @author Jörg Steffen, DFKI
 */
public class Board {

  private int[][] numbers;
  private boolean[][] marks;

  /**
   * Create a new board from the given lines.
   *
   * @param lines with one board row per line
   */
  public Board(List<String> lines) {
    this.numbers = new int[lines.size()][];
    this.marks = new boolean[lines.size()][];
    for (int i = 0; i < lines.size(); i++) {
      String oneLine = lines.get(i);
      String[] parts = oneLine.strip().split("\\s+");
      this.numbers[i] = new int[parts.length];
      this.marks[i] = new boolean[parts.length];
      for (int j = 0; j < parts.length; j++) {
        this.numbers[i][j] = Integer.parseInt(parts[j]);
      }
    }
  }

  /**
   * Check if given number is on the board and mark it if found.
   *
   * @param num the number to check
   */
  public void checkNumber(int num) {
    for (int i = 0; i < this.numbers.length; i++) {
      for (int j = 0; j < this.numbers[i].length; j++) {
        if (this.numbers[i][j] == num) {
          this.marks[i][j] = true;
          return;
        }
      }
    }
  }

  /**
   * Check if the board has a complete row or column of marked numbers.
   *
   * @return {@code true} if complete row/column is found, {@code false} otherwise
   */
  public boolean hasWon() {
    return hasCompleteRow() || hasCompleteColumn();
  }

  private boolean hasCompleteRow() {
    outer:
    for (int i = 0; i < this.numbers.length; i++) {
      for (int j = 0; j < this.numbers[i].length; j++) {
        if (!this.marks[i][j]) {
          continue outer;
        }
      }
      return true;
    }
    return false;
  }

  private boolean hasCompleteColumn() {
    outer:
    for (int i = 0; i < this.numbers[0].length; i++) {
      for (int j = 0; j < this.numbers.length; j++) {
        if (!this.marks[j][i]) {
          continue outer;
        }
      }
      return true;
    }
    return false;
  }

  /**
   * Return sum of unmarked numbers.
   *
   * @return the sum
   */
  public int getUnmarkedNumbersSum() {
    int result = 0;
    for (int i = 0; i < this.numbers.length; i++) {
      for (int j = 0; j < this.numbers[i].length; j++) {
        if (!this.marks[i][j]) {
          result += this.numbers[i][j];
        }
      }
    }
    return result;
  }

  @Override
  public String toString() {
    StringBuilder result = new StringBuilder();
    for (int i = 0; i < this.numbers.length; i++) {
      for (int j = 0; j < numbers[i].length; j++) {
        result.append(String.format("%2d%s ", numbers[i][j], marks[i][j] ? "*" : " "));
      }
      result.append(String.format("%n"));
    }
    return result.toString();
  }
}
