package de.unisaar.jfap.ex5.task3;

import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Directory with files and nested directories.
 *
 * @author Jörg Steffen, DFKI
 */
public class Directory implements Comparable<Directory> {

  private static final Logger logger = LoggerFactory.getLogger(Directory.class);

  private String path;
  private List<File> files;
  private List<Directory> directories;
  private int size;

  /**
   * Create a new empty directory having the given path.
   *
   * @param path the path
   */
  public Directory(String path) {
    logger.debug("creating new directory {}", path);
    this.path = path;
    this.files = new ArrayList<>();
    this.directories = new ArrayList<>();
    this.size = -1;
  }

  public String getPath() {
    return this.path;
  }

  public List<File> getFiles() {
    return this.files;
  }

  public List<Directory> getDirectories() {
    return this.directories;
  }

  public void addFile(File file) {
    this.files.add(file);
  }

  public void addDirectory(Directory dir) {
    this.directories.add(dir);
  }

  /**
   * Get size of the directory. This includes the size of all contained files and directories.
   *
   * @return the directories size
   */
  public int getSize() {
    if (this.size == -1) {
      this.size = 0;
      for (File oneFile : this.files) {
        this.size += oneFile.getSize();
      }
      for (Directory oneDir : this.directories) {
        this.size += oneDir.getSize();
      }
    }
    return size;
  }

  @Override
  public int compareTo(Directory o) {
    return Integer.compare(o.getSize(), this.getSize());
  }

  @Override
  public String toString() {
    return String.format("%s: %,d", this.path, this.size);
  }
}
